/*
 * Copyright (C) 2018-2022 52°North Spatial Information Research GmbH
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation.
 *
 * If the program is linked with libraries which are licensed under one of
 * the following licenses, the combination of the program with the linked
 * library is not considered a "derivative work" of the program:
 *
 *     - Apache License, version 2.0
 *     - Apache Software License, version 1.0
 *     - GNU Lesser General Public License, version 3
 *     - Mozilla Public License, versions 1.0, 1.1 and 2.0
 *     - Common Development and Distribution License (CDDL), version 1.0
 *
 * Therefore the distribution of the program linked with libraries licensed
 * under the aforementioned licenses, is permitted by the copyright holders
 * if the distribution is compliant with both the GNU General Public
 * License version 2 and the aforementioned licenses.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 */
//@SpringBootConfiguration
//@EnableAutoConfiguration
//@AutoConfigureTestDatabase
//@DataJpaTest

package org.n52.sta.conformance;

import java.io.IOException;

import com.fasterxml.jackson.databind.JsonNode;
import com.fasterxml.jackson.databind.ObjectMapper;

import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpUriRequest;
import org.apache.http.entity.ContentType;
import org.apache.http.impl.client.HttpClientBuilder;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.test.context.TestPropertySource;
import org.testcontainers.junit.jupiter.Testcontainers;

/**
 * This class tests various different things not covered directly by the official cite tests
 */
@Testcontainers
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.DEFINED_PORT)
@TestPropertySource(locations = "classpath:application-test.properties")
public class ITRootResponseTest extends ConformanceTests {

    protected final static String jsonMimeType = "application/json";

    private ObjectMapper mapper = new ObjectMapper();

    ITRootResponseTest(@Value("${server.config.service-root-url}") String rootUrl) {
        super(rootUrl);
    }

    @Test
    public void rootResponseIsCorrect() throws IOException {
        HttpUriRequest request = new HttpGet(rootUrl);
        HttpResponse response = HttpClientBuilder.create()
                                                 .build()
                                                 .execute(request);

        // Check Response MIME Type
        String mimeType = ContentType.getOrDefault(response.getEntity())
                                     .getMimeType();
        Assertions.assertEquals(jsonMimeType, mimeType);

        // Check Response
        JsonNode root = mapper.readTree(response.getEntity()
                                                .getContent());
        Assertions.assertTrue(root.hasNonNull("value"));

        // TODO: expand tests to actually be useful
    }

}
