/*
 * Copyright (C) 2018-2022 52°North Spatial Information Research GmbH
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License version 2 as published
 * by the Free Software Foundation.
 *
 * If the program is linked with libraries which are licensed under one of
 * the following licenses, the combination of the program with the linked
 * library is not considered a "derivative work" of the program:
 *
 *     - Apache License, version 2.0
 *     - Apache Software License, version 1.0
 *     - GNU Lesser General Public License, version 3
 *     - Mozilla Public License, versions 1.0, 1.1 and 2.0
 *     - Common Development and Distribution License (CDDL), version 1.0
 *
 * Therefore the distribution of the program linked with libraries licensed
 * under the aforementioned licenses, is permitted by the copyright holders
 * if the distribution is compliant with both the GNU General Public
 * License version 2 and the aforementioned licenses.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 */

package org.n52.sta.data.entity;

import java.util.Map;
import java.util.Optional;
import java.util.Set;
import java.util.function.Predicate;
import java.util.stream.Stream;

import org.n52.janmayen.stream.Streams;
import org.n52.series.db.beans.FormatEntity;
import org.n52.series.db.beans.ProcedureEntity;
import org.n52.series.db.beans.ProcedureHistoryEntity;
import org.n52.sta.api.entity.Datastream;
import org.n52.sta.api.entity.Sensor;
import org.n52.sta.config.EntityPropertyMapping;

public class SensorData extends StaData<ProcedureEntity> implements Sensor {

    public SensorData(ProcedureEntity dataEntity, Optional<EntityPropertyMapping> propertyMapping) {
        super(dataEntity, propertyMapping);
    }

    @Override
    public String getName() {
        return data.getName();
    }

    @Override
    public String getDescription() {
        return data.getDescription();
    }

    @Override
    public String getEncodingType() {
        FormatEntity format = data.getFormat();
        return format.getFormat();
    }

    @Override
    public String getMetadata() {
        String file = data.getDescriptionFile();
        return useLinkToFile(file).orElse(getXml(data).orElse(null));
    }

    @Override
    public Map<String, Object> getProperties() {
        return toMap(data.getParameters());
    }

    @Override
    public Set<Datastream> getDatastreams() {
        return toSet(data.getDatasets(), entity -> new DatastreamData(entity, propertyMapping));
    }

    private Optional<String> useLinkToFile(String file) {
        return Optional.ofNullable(file)
                       .filter(f -> !f.isEmpty());
    }

    private Optional<String> getXml(ProcedureEntity entity) {
        Set<ProcedureHistoryEntity> history = entity.getProcedureHistory();
        return findCurrentFrom(history).map(ProcedureHistoryEntity::getXml)
                                       .findFirst();
    }

    private Stream<ProcedureHistoryEntity> findCurrentFrom(Set<ProcedureHistoryEntity> history) {
        return Streams.stream(history)
                      .filter(isLatest());
    }

    private Predicate< ? super ProcedureHistoryEntity> isLatest() {
        return entity -> entity.getEndTime() == null;
    }

}
